@@ -14,10 +14,6 @@
 # ==============================================================================
 """Tests for utilities working with arbitrarily nested structures."""
 
-from __future__ import absolute_import
-from __future__ import division
-from __future__ import print_function
-
 import collections
 import doctest
 import types
@@ -26,7 +22,6 @@ import unittest
 from absl.testing import parameterized
 import attr
 import numpy as np
-import six
 import tree
 import wrapt
 
@@ -992,14 +987,34 @@ class NestTest(parameterized.TestCase):
       tree.map_structure_with_path(lambda path, *s: 0, s1, s2)
 
   def testMappingProxyType(self):
-    if six.PY2:
-      self.skipTest("Python 2 does not support mapping proxy type.")
-
     structure = types.MappingProxyType({"a": 1, "b": (2, 3)})
     expected = types.MappingProxyType({"a": 4, "b": (5, 6)})
     self.assertEqual(tree.flatten(structure), [1, 2, 3])
     self.assertEqual(tree.unflatten_as(structure, [4, 5, 6]), expected)
     self.assertEqual(tree.map_structure(lambda v: v + 3, structure), expected)
 
+  def testTraverseListsToTuples(self):
+    structure = [(1, 2), [3], {"a": [4]}]
+    self.assertEqual(
+        ((1, 2), (3,), {"a": (4,)}),
+        tree.traverse(
+            lambda x: tuple(x) if isinstance(x, list) else x,
+            structure,
+            top_down=False))
+
+  def testTraverseEarlyTermination(self):
+    structure = [(1, [2]), [3, (4, 5, 6)]]
+    visited = []
+    def visit(x):
+      visited.append(x)
+      return "X" if isinstance(x, tuple) and len(x) > 2 else None
+
+    output = tree.traverse(visit, structure)
+    self.assertEqual([(1, [2]), [3, "X"]], output)
+    self.assertEqual(
+        [[(1, [2]), [3, (4, 5, 6)]],
+         (1, [2]), 1, [2], 2, [3, (4, 5, 6)], 3, (4, 5, 6)],
+        visited)
+
 if __name__ == "__main__":
   unittest.main()
